@@ -549,6 +549,12 @@ public:
   #define PETSC_PKG_CUDA_VERSION_GE(...) 0
   #define CUPM_DEFINED_PETSC_PKG_CUDA_VERSION_GE
 #endif
+
+#if !defined(PETSC_PKG_HIP_VERSION_LT)
+  #define PETSC_PKG_HIP_VERSION_LT(...) 0
+  #define CUPM_DEFINED_PETSC_PKG_HIP_VERSION_LT
+#endif
+
   static PetscErrorCode PetscCUPMGetMemType(const void *data, PetscMemType *type, PetscBool *registered = nullptr, PetscBool *managed = nullptr) noexcept
   {
     cupmPointerAttributes_t attr;
@@ -569,14 +575,14 @@ public:
     cerr = cupmPointerGetAttributes(&attr, data);
     cerr = cupmGetLastError();
     // HIP seems to always have used memoryType though
-#if (defined(CUDART_VERSION) && (CUDART_VERSION < 10000)) || defined(__HIP_PLATFORM_HCC__)
+#if (defined(CUDART_VERSION) && (CUDART_VERSION < 10000)) || (defined(__HIP_PLATFORM_HCC__) && PETSC_PKG_HIP_VERSION_LT(5, 5, 0))
     const auto mtype = attr.memoryType;
     if (managed) *managed = static_cast<PetscBool>((cerr == cupmSuccess) && attr.isManaged);
 #else
     if (PETSC_PKG_CUDA_VERSION_GE(11, 0, 0) && (T == DeviceType::CUDA)) PetscCallCUPM(cerr);
     const auto mtype = attr.type;
     if (managed) *managed = static_cast<PetscBool>(mtype == cupmMemoryTypeManaged);
-#endif // CUDART_VERSION && CUDART_VERSION < 10000 || __HIP_PLATFORM_HCC__
+#endif // CUDART_VERSION && CUDART_VERSION < 10000 || (defined(__HIP_PLATFORM_HCC__) && PETSC_PKG_HIP_VERSION_LT(5, 5, 0))
     if (type) *type = ((cerr == cupmSuccess) && (mtype == cupmMemoryTypeDevice)) ? PETSC_MEMTYPE_CUPM() : PETSC_MEMTYPE_HOST;
     if (registered && (cerr == cupmSuccess) && (mtype == cupmMemoryTypeHost)) *registered = PETSC_TRUE;
     PetscFunctionReturn(PETSC_SUCCESS);
@@ -584,6 +590,9 @@ public:
 #if defined(CUPM_DEFINED_PETSC_PKG_CUDA_VERSION_GE)
   #undef PETSC_PKG_CUDA_VERSION_GE
 #endif
+#if defined(CUPM_DEFINED_PETSC_PKG_HIP_VERSION_LT)
+  #undef PETSC_PKG_HIP_VERSION_LT
+#endif
 
   PETSC_NODISCARD static PETSC_CONSTEXPR_14 cupmMemcpyKind_t PetscDeviceCopyModeToCUPMMemcpyKind(PetscDeviceCopyMode mode) noexcept
   {
@@ -838,7 +847,7 @@ public:
     // if n = 0 then nthread = 0, which is not allowed. rather than letting the user try to
     // decipher cryptic 'cuda/hipErrorLaunchFailure' we explicitly check for zero here
     PetscAssert(nthread, PETSC_COMM_SELF, PETSC_ERR_ARG_WRONG, "Trying to launch kernel with grid/block size 0");
-    PetscCallCUPM(cupmLaunchKernel(std::forward<F>(func), nblock, nthread, sharedMem, stream, std::forward<Args>(kernelArgs)...));
+    PetscCallCUPM(cupmLaunchKernel(std::forward<F>(func), (unsigned int)nblock, (unsigned int)nthread, sharedMem, stream, std::forward<Args>(kernelArgs)...));
     PetscFunctionReturn(PETSC_SUCCESS);
   }
 
