@@ -17,6 +17,7 @@ typedef struct {
   Vec              work1;
   Vec              work2;
   PetscScalar      alpha;
+  Mat              alpha_mat;
 } PC_Composite;
 
 static PetscErrorCode PCApply_Composite_Multiplicative(PC pc, Vec x, Vec y)
@@ -26,7 +27,6 @@ static PetscErrorCode PCApply_Composite_Multiplicative(PC pc, Vec x, Vec y)
   Mat              mat  = pc->pmat;
 
   PetscFunctionBegin;
-
   PetscCheck(next, PetscObjectComm((PetscObject)pc), PETSC_ERR_ARG_WRONGSTATE, "No composite preconditioners supplied via PCCompositeAddPCType() or -pc_composite_pcs");
 
   /* Set the reuse flag on children PCs */
@@ -97,8 +97,8 @@ static PetscErrorCode PCApplyTranspose_Composite_Multiplicative(PC pc, Vec x, Ve
 
 /*
     This is very special for a matrix of the form alpha I + R + S
-where first preconditioner is built from alpha I + S and second from
-alpha I + R
+    where first preconditioner is built from alpha I + S and second from
+    alpha I + R
 */
 static PetscErrorCode PCApply_Composite_Special(PC pc, Vec x, Vec y)
 {
@@ -114,7 +114,11 @@ static PetscErrorCode PCApply_Composite_Special(PC pc, Vec x, Vec y)
   PetscCall(PCSetReusePreconditioner(next->next->pc, pc->reusepreconditioner));
 
   PetscCall(PCApply(next->pc, x, jac->work1));
-  PetscCall(PCApply(next->next->pc, jac->work1, y));
+  if (jac->alpha_mat) {
+    if (!jac->work2) PetscCall(VecDuplicate(jac->work1, &jac->work2));
+    PetscCall(MatMult(jac->alpha_mat, jac->work1, jac->work2));
+    PetscCall(PCApply(next->next->pc, jac->work2, y));
+  } else PetscCall(PCApply(next->next->pc, jac->work1, y));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -184,7 +188,7 @@ static PetscErrorCode PCSetUpOnBlocks_Composite(PC pc)
   PetscFunctionBegin;
   while (next) {
     PetscCall(PCSetUp(next->pc));
-    PetscCall(PCGetFailedReasonRank(next->pc, &reason));
+    PetscCall(PCGetFailedReason(next->pc, &reason));
     if (reason) pc->failedreason = reason;
     next = next->next;
   }
@@ -203,6 +207,7 @@ static PetscErrorCode PCReset_Composite(PC pc)
   }
   PetscCall(VecDestroy(&jac->work1));
   PetscCall(VecDestroy(&jac->work2));
+  PetscCall(MatDestroy(&jac->alpha_mat));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -226,22 +231,24 @@ static PetscErrorCode PCDestroy_Composite(PC pc)
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeGetNumberPC_C", NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeGetPC_C", NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeSpecialSetAlpha_C", NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeSpecialSetAlphaMat_C", NULL));
   PetscCall(PetscFree(pc->data));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode PCSetFromOptions_Composite(PC pc, PetscOptionItems *PetscOptionsObject)
 {
-  PC_Composite    *jac  = (PC_Composite *)pc->data;
-  PetscInt         nmax = 8, i;
+  PC_Composite    *jac = (PC_Composite *)pc->data;
+  PetscInt         nmax, i;
   PC_CompositeLink next;
-  char            *pcs[8];
+  char            *pcs[1024];
   PetscBool        flg;
 
   PetscFunctionBegin;
   PetscOptionsHeadBegin(PetscOptionsObject, "Composite preconditioner options");
   PetscCall(PetscOptionsEnum("-pc_composite_type", "Type of composition", "PCCompositeSetType", PCCompositeTypes, (PetscEnum)jac->type, (PetscEnum *)&jac->type, &flg));
   if (flg) PetscCall(PCCompositeSetType(pc, jac->type));
+  nmax = (PetscInt)PETSC_STATIC_ARRAY_LENGTH(pcs);
   PetscCall(PetscOptionsStringArray("-pc_composite_pcs", "List of composite solvers", "PCCompositeAddPCType", pcs, &nmax, &flg));
   if (flg) {
     for (i = 0; i < nmax; i++) {
@@ -293,6 +300,20 @@ static PetscErrorCode PCCompositeSpecialSetAlpha_Composite(PC pc, PetscScalar al
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
+static PetscErrorCode PCCompositeSpecialSetAlphaMat_Composite(PC pc, Mat alpha_mat)
+{
+  PC_Composite *jac = (PC_Composite *)pc->data;
+
+  PetscFunctionBegin;
+  if (alpha_mat) {
+    PetscValidHeaderSpecific(alpha_mat, MAT_CLASSID, 2);
+    PetscCall(PetscObjectReference((PetscObject)alpha_mat));
+  }
+  PetscCall(MatDestroy(&jac->alpha_mat));
+  jac->alpha_mat = alpha_mat;
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
 static PetscErrorCode PCCompositeSetType_Composite(PC pc, PCCompositeType type)
 {
   PC_Composite *jac = (PC_Composite *)pc->data;
@@ -417,7 +438,7 @@ static PetscErrorCode PCCompositeGetPC_Composite(PC pc, PetscInt n, PC *subpc)
 
   Level: advanced
 
-.seealso: `PCCOMPOSITE`, `PC_COMPOSITE_ADDITIVE`, `PC_COMPOSITE_MULTIPLICATIVE`, `PC_COMPOSITE_SPECIAL`, `PCCompositeType`,
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PC_COMPOSITE_ADDITIVE`, `PC_COMPOSITE_MULTIPLICATIVE`, `PC_COMPOSITE_SPECIAL`, `PCCompositeType`,
           `PCCompositeGetType()`
 @*/
 PetscErrorCode PCCompositeSetType(PC pc, PCCompositeType type)
@@ -442,7 +463,7 @@ PetscErrorCode PCCompositeSetType(PC pc, PCCompositeType type)
 
   Level: advanced
 
-.seealso: `PCCOMPOSITE`, `PC_COMPOSITE_ADDITIVE`, `PC_COMPOSITE_MULTIPLICATIVE`, `PC_COMPOSITE_SPECIAL`, `PCCompositeType`,
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PC_COMPOSITE_ADDITIVE`, `PC_COMPOSITE_MULTIPLICATIVE`, `PC_COMPOSITE_SPECIAL`, `PCCompositeType`,
           `PCCompositeSetType()`
 @*/
 PetscErrorCode PCCompositeGetType(PC pc, PCCompositeType *type)
@@ -455,7 +476,7 @@ PetscErrorCode PCCompositeGetType(PC pc, PCCompositeType *type)
 
 /*@
   PCCompositeSpecialSetAlpha - Sets alpha for the special composite preconditioner, `PC_COMPOSITE_SPECIAL`,
-  for alphaI + R + S
+  for $\alpha I + R + S$
 
   Logically Collective
 
@@ -465,7 +486,7 @@ PetscErrorCode PCCompositeGetType(PC pc, PCCompositeType *type)
 
   Level: developer
 
-.seealso: `PCCOMPOSITE`, `PC_COMPOSITE_ADDITIVE`, `PC_COMPOSITE_MULTIPLICATIVE`, `PC_COMPOSITE_SPECIAL`, `PCCompositeType`,
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PC_COMPOSITE_ADDITIVE`, `PC_COMPOSITE_MULTIPLICATIVE`, `PC_COMPOSITE_SPECIAL`, `PCCompositeType`,
           `PCCompositeSetType()`, `PCCompositeGetType()`
 @*/
 PetscErrorCode PCCompositeSpecialSetAlpha(PC pc, PetscScalar alpha)
@@ -477,6 +498,14 @@ PetscErrorCode PCCompositeSpecialSetAlpha(PC pc, PetscScalar alpha)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
+PetscErrorCode PCCompositeSpecialSetAlphaMat(PC pc, Mat alpha_mat)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc, PC_CLASSID, 1);
+  PetscTryMethod(pc, "PCCompositeSpecialSetAlphaMat_C", (PC, Mat), (pc, alpha_mat));
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
 /*@C
   PCCompositeAddPCType - Adds another `PC` of the given type to the composite `PC`.
 
@@ -488,7 +517,7 @@ PetscErrorCode PCCompositeSpecialSetAlpha(PC pc, PetscScalar alpha)
 
   Level: intermediate
 
-.seealso: `PCCOMPOSITE`, `PCCompositeAddPC()`, `PCCompositeGetNumberPC()`
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PCCompositeAddPC()`, `PCCompositeGetNumberPC()`
 @*/
 PetscErrorCode PCCompositeAddPCType(PC pc, PCType type)
 {
@@ -509,7 +538,7 @@ PetscErrorCode PCCompositeAddPCType(PC pc, PCType type)
 
   Level: intermediate
 
-.seealso: `PCCOMPOSITE`, `PCCompositeAddPCType()`, `PCCompositeGetNumberPC()`
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PCCompositeAddPCType()`, `PCCompositeGetNumberPC()`
 @*/
 PetscErrorCode PCCompositeAddPC(PC pc, PC subpc)
 {
@@ -533,7 +562,7 @@ PetscErrorCode PCCompositeAddPC(PC pc, PC subpc)
 
   Level: developer
 
-.seealso: `PCCOMPOSITE`, `PCCompositeGetPC()`, `PCCompositeAddPC()`, `PCCompositeAddPCType()`
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PCCompositeGetPC()`, `PCCompositeAddPC()`, `PCCompositeAddPCType()`
 @*/
 PetscErrorCode PCCompositeGetNumberPC(PC pc, PetscInt *num)
 {
@@ -562,7 +591,7 @@ PetscErrorCode PCCompositeGetNumberPC(PC pc, PetscInt *num)
   To use a different operator to construct one of the inner preconditioners first call `PCCompositeGetPC()`, then
   call `PCSetOperators()` on that `PC`.
 
-.seealso: `PCCOMPOSITE`, `PCCompositeAddPCType()`, `PCCompositeGetNumberPC()`, `PCSetOperators()`
+.seealso: [](ch_ksp), `PCCOMPOSITE`, `PCCompositeAddPCType()`, `PCCompositeGetNumberPC()`, `PCSetOperators()`
 @*/
 PetscErrorCode PCCompositeGetPC(PC pc, PetscInt n, PC *subpc)
 {
@@ -578,8 +607,8 @@ PetscErrorCode PCCompositeGetPC(PC pc, PetscInt n, PC *subpc)
 
    Options Database Keys:
 +  -pc_composite_type <type: one of multiplicative, additive, symmetric_multiplicative, special> - Sets composite preconditioner type
-.  -pc_use_amat - activates `PCSetUseAmat()`
--  -pc_composite_pcs - <pc0,pc1,...> list of PCs to compose
+.  -pc_use_amat                                                                                  - activates `PCSetUseAmat()`
+-  -pc_composite_pcs                                                                             - <pc0,pc1,...> list of PCs to compose
 
    Level: intermediate
 
@@ -591,7 +620,7 @@ PetscErrorCode PCCompositeGetPC(PC pc, PetscInt n, PC *subpc)
    To use a different operator to construct one of the inner preconditioners first call `PCCompositeGetPC()`, then
    call `PCSetOperators()` on that `PC`.
 
-.seealso: `PCCreate()`, `PCSetType()`, `PCType`, `PC`,
+.seealso: [](ch_ksp), `PCCreate()`, `PCSetType()`, `PCType`, `PC`,
           `PCSHELL`, `PCKSP`, `PCCompositeSetType()`, `PCCompositeSpecialSetAlpha()`, `PCCompositeAddPCType()`,
           `PCCompositeGetPC()`, `PCSetUseAmat()`, `PCCompositeAddPC()`, `PCCompositeGetNumberPC()`
 M*/
@@ -613,11 +642,12 @@ PETSC_EXTERN PetscErrorCode PCCreate_Composite(PC pc)
   pc->ops->view            = PCView_Composite;
   pc->ops->applyrichardson = NULL;
 
-  pc->data   = (void *)jac;
-  jac->type  = PC_COMPOSITE_ADDITIVE;
-  jac->work1 = NULL;
-  jac->work2 = NULL;
-  jac->head  = NULL;
+  pc->data       = (void *)jac;
+  jac->type      = PC_COMPOSITE_ADDITIVE;
+  jac->work1     = NULL;
+  jac->work2     = NULL;
+  jac->head      = NULL;
+  jac->alpha_mat = NULL;
 
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeSetType_C", PCCompositeSetType_Composite));
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeGetType_C", PCCompositeGetType_Composite));
@@ -626,5 +656,6 @@ PETSC_EXTERN PetscErrorCode PCCreate_Composite(PC pc)
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeGetNumberPC_C", PCCompositeGetNumberPC_Composite));
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeGetPC_C", PCCompositeGetPC_Composite));
   PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeSpecialSetAlpha_C", PCCompositeSpecialSetAlpha_Composite));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pc, "PCCompositeSpecialSetAlphaMat_C", PCCompositeSpecialSetAlphaMat_Composite));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
